@@ -1,13 +1,13 @@
 seg.def.fit.boot<-function(obj, Z, PSI, mfExt, opz, n.boot=10, size.boot=NULL, jt=FALSE,
-    nonParam=TRUE, random=FALSE){
+    nonParam=TRUE, random=FALSE, break.boot=n.boot){
 #random se TRUE prende valori random quando e' errore: comunque devi modificare qualcosa (magari con it.max)
 #     per fare restituire la dev in corrispondenza del punto psi-random
 #nonParm. se TRUE implemneta il case resampling. Quello semiparam dipende dal non-errore di
 extract.psi<-function(lista){
 #serve per estrarre il miglior psi..
-    	dev.values<-lista[[1]]
-    	psi.values<-lista[[2]]
-    	dev.ok<-min(dev.values)
+  dev.values<-lista[[1]][-1] #remove the 1st one referring to model without psi
+  psi.values<-lista[[2]][-1] #remove the 1st one (NA)
+  dev.ok<-min(dev.values)
     	id.dev.ok<-which.min(dev.values)
     	if(is.list(psi.values))  psi.values<-matrix(unlist(psi.values),
     		nrow=length(dev.values), byrow=TRUE)
@@ -19,12 +19,13 @@ extract.psi<-function(lista){
 #-------------
       visualBoot<-opz$visualBoot
       opz.boot<-opz
-      opz.boot$pow=c(1.1,1.2)
+      opz.boot$pow=c(1,1) #c(1.1,1.2)
       opz1<-opz
       opz1$it.max <-1
       n<-nrow(mfExt)
-      o0<-try(seg.def.fit(obj, Z, PSI, mfExt, opz), silent=TRUE)
+      o0<-try(suppressWarnings(seg.def.fit(obj, Z, PSI, mfExt, opz)), silent=TRUE)
       rangeZ <- apply(Z, 2, range) #serve sempre
+      
       if(!is.list(o0)) {
           o0<- seg.def.fit(obj, Z, PSI, mfExt, opz, return.all.sol=TRUE)
           o0<-extract.psi(o0)
@@ -39,14 +40,15 @@ extract.psi<-function(lista){
           if(random) {
             est.psi00<-est.psi0<-apply(rangeZ,2,function(r)runif(1,r[1],r[2]))
             PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
-            o0<-try(seg.def.fit(obj, Z, PSI1, mfExt, opz1), silent=TRUE)
+            o0<-try(suppressWarnings(seg.def.fit(obj, Z, PSI1, mfExt, opz1)), silent=TRUE)
             ss00<-o0$SumSquares.no.gap
           } else {
           est.psi00<-est.psi0<-apply(PSI,2,mean)
           ss00<-opz$dev0
         }
         }
-
+      n.intDev0<-nchar(strsplit(as.character(ss00),"\\.")[[1]][1])
+      
       all.est.psi.boot<-all.selected.psi<-all.est.psi<-matrix(, nrow=n.boot, ncol=length(est.psi0))
       all.ss<-all.selected.ss<-rep(NA, n.boot)
       if(is.null(size.boot)) size.boot<-n
@@ -54,20 +56,30 @@ extract.psi<-function(lista){
 #      na<- ,,apply(...,2,function(x)mean(is.na(x)))
 
       Z.orig<-Z
-      if(visualBoot) cat(0, " ", formatC(opz$dev0, 3, format = "f"),"", "(No breakpoint(s))", "\n")
+#      if(visualBoot) cat(0, " ", formatC(opz$dev0, 3, format = "f"),"", "(No breakpoint(s))", "\n")
       count.random<-0
       for(k in seq(n.boot)){
+        ##se gli *ultimi* n.boot.rev valori di ss sono uguali, cambia i psi...
+        n.boot.rev<- 3 #3 o 4?
+        diff.selected.ss <- rev(diff(na.omit(all.selected.ss)))
+        #if(length(na.omit(diff(all.selected.ss[1:n.boot.rev])))==(n.boot.rev-1) && all(round(diff(all.selected.ss[1:n.boot.rev]),6)==0)){
+        if(length(diff.selected.ss)>=(n.boot.rev-1) && all(round(diff.selected.ss[1:(n.boot.rev-1)],6)==0)){
+          qpsi<-sapply(1:ncol(Z),function(i)mean(est.psi0[i]>=Z[,i]))
+          qpsi<-ifelse(abs(qpsi-.5)<.1,.8,qpsi)
+          est.psi0<-sapply(1:ncol(Z),function(i)quantile(Z[,i],probs=1-qpsi[i],names=FALSE))
+        }
+        
           PSI <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
           if(jt) Z<-apply(Z.orig,2,jitter)
           if(nonParam){
               id<-sample(n, size=size.boot, replace=TRUE)
-              o.boot<-try(seg.def.fit(obj, Z[id,,drop=FALSE], PSI[id,,drop=FALSE], mfExt[id,,drop=FALSE], opz.boot), silent=TRUE)
+              o.boot<-try(suppressWarnings(seg.def.fit(obj, Z[id,,drop=FALSE], PSI[id,,drop=FALSE], mfExt[id,,drop=FALSE], opz.boot)), silent=TRUE)
          
           } else {
               yy<-fitted.ok+sample(residuals(o0),size=n, replace=TRUE)
 ##---->              o.boot<-try(seg.lm.fit(yy, XREG, Z.orig, PSI, weights, offs, opz.boot), silent=TRUE)
                     #in realta' la risposta dovrebbe essere "yy" da cambiare in mfExt
-                    o.boot<- try(seg.def.fit(obj, Z.orig, PSI, mfExt, opz.boot), silent=TRUE)
+                    o.boot<- try(suppressWarnings(seg.def.fit(obj, Z.orig, PSI, mfExt, opz.boot)), silent=TRUE)
           }
           if(is.list(o.boot)){
             all.est.psi.boot[k,]<-est.psi.boot<-o.boot$psi
@@ -77,15 +89,16 @@ extract.psi<-function(lista){
             PSI <- matrix(rep(est.psi.boot, rep(nrow(Z), length(est.psi.boot))), ncol = length(est.psi.boot))
             opz$h<-max(opz$h*.9, .2)
             opz$it.max<-opz$it.max+1
-                  o <- try(seg.def.fit(obj, Z.orig, PSI, mfExt, opz, return.all.sol=TRUE), silent=TRUE)
+            o <- try(seg.def.fit(obj, Z.orig, PSI, mfExt, opz, return.all.sol=TRUE), silent=TRUE)
             if(!is.list(o) && random){
                 est.psi0<-apply(rangeZ,2,function(r)runif(1,r[1],r[2]))
                 PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
-            o <- try(seg.def.fit(obj, Z, PSI1, mfExt, opz1), silent=TRUE)
+            o <- try(suppressWarnings(seg.def.fit(obj, Z, PSI1, mfExt, opz1)), silent=TRUE)
                 count.random<-count.random+1
               }
             if(is.list(o)){
-              if(!"coefficients"%in%names(o$obj)) o<-extract.psi(o)
+              #if(!"coefficients"%in%names(o$obj)) o<-extract.psi(o)
+              if( isFALSE( "coefficients"%in%names(o$obj) || "estimate"%in%names(o$obj) ) ) o<-extract.psi(o)
               all.est.psi[k,]<-o$psi
               all.ss[k]<-o$SumSquares.no.gap
               if(o$SumSquares.no.gap<=ifelse(is.list(o0), o0$SumSquares.no.gap, 10^12)) o0<-o
@@ -93,12 +106,22 @@ extract.psi<-function(lista){
               all.selected.psi[k,] <- est.psi0
               all.selected.ss[k]<-o0$SumSquares.no.gap #min(c(o$SumSquares.no.gap, o0$SumSquares.no.gap))
               }
-            if(visualBoot) {
+            if (visualBoot) {
               flush.console()
-              spp <- if (k < 10) "" else NULL
-              cat(k, spp, "", formatC(o0$SumSquares.no.gap, 3, format = "f"), "\n")
-              }
-            } #end n.boot
+              cat(paste("boot sample = ", sprintf("%2.0f",k),
+                        "  opt.min.f = ", sprintf(paste("%", n.intDev0+6, ".5f",sep=""), o0$SumSquares.no.gap), #formatC(L1,width=8, digits=5,format="f"), #era format="fg" 
+                        "  n.psi = ",formatC(length(unlist(est.psi0)),digits=0,format="f"), 
+                        "  est.psi = ",paste(formatC(unlist(est.psi0),digits=3,format="f"), collapse="  "), #sprintf('%.2f',x)
+                        sep=""), "\n")
+            }
+            asss<-na.omit(all.selected.ss)
+            if(length(asss)>break.boot){
+              if(all(rev(round(diff(asss),6))[1:(break.boot-1)]==0)) break
+            }
+            
+          } #end n.boot
+
+      
       all.selected.psi<-rbind(est.psi00,all.selected.psi)
       all.selected.ss<-c(ss00, all.selected.ss)
 
@@ -106,9 +129,10 @@ extract.psi<-function(lista){
 
       if(is.null(o0$obj)){
           PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
-            o0 <- try(seg.def.fit(obj, Z, PSI1, mfExt, opz1), silent=TRUE)
+            o0 <- try(suppressWarnings(seg.def.fit(obj, Z, PSI1, mfExt, opz1)), silent=TRUE)
       }
       if(!is.list(o0)) return(0)
       o0$boot.restart<-ris
+      rm(.Random.seed, envir=globalenv())
       return(o0)
       }
\ No newline at end of file
