@@ -1,5 +1,5 @@
 seg.lm.fit.boot <- function(y, XREG, Z, PSI, w, offs, opz, n.boot=10, size.boot=NULL, jt=FALSE,
-    nonParam=TRUE, random=FALSE){
+    nonParam=TRUE, random=FALSE, break.boot=n.boot){
 #random se TRUE prende valori random quando e' errore: comunque devi modificare qualcosa (magari con it.max)
 #     per fare restituire la dev in corrispondenza del punto psi-random
 #nonParm. se TRUE implemneta il case resampling. Quello semiparam dipende dal non-errore di
@@ -69,51 +69,68 @@ extract.psi<-function(lista){
       all.est.psi.boot<-all.selected.psi<-all.est.psi<-matrix(NA, nrow=n.boot, ncol=length(est.psi0))
       all.ss<-all.selected.ss<-rep(NA, n.boot)
       if(is.null(size.boot)) size.boot<-n
-      
 #      na<- ,,apply(...,2,function(x)mean(is.na(x)))
-
       Z.orig<-Z
-#      if(visualBoot) cat(0, " ", formatC(opz$dev0, 3, format = "f"),"", "(No breakpoint(s))", "\n")
+#     if(visualBoot) cat(0, " ", formatC(opz$dev0, 3, format = "f"),"", "(No breakpoint(s))", "\n")
       count.random<-0
+      id.uguali<-0
+      k.psi.change<- 1
+      alpha<-.1
       for(k in seq(n.boot)){
-          PSI <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
-          if(jt) Z<-apply(Z.orig,2,jitter)
-          if(nonParam){
+        #if(k==5) browser()
+        ##se gli *ultimi* n.boot.rev valori di ss sono uguali, cambia i psi...
+        n.boot.rev<- 3 #3 o 4?
+        diff.selected.ss <- rev(diff(na.omit(all.selected.ss)))
+        #if(length(na.omit(diff(all.selected.ss[1:n.boot.rev])))==(n.boot.rev-1) && all(round(diff(all.selected.ss[1:n.boot.rev]),6)==0)){
+        if(length(diff.selected.ss)>=(n.boot.rev-1) && all(round(diff.selected.ss[1:(n.boot.rev-1)],6)==0)){
+          qpsi<-sapply(1:ncol(Z),function(i)mean(est.psi0[i]>=Z[,i]))
+          qpsi<-ifelse(abs(qpsi-.5)<.1, alpha, qpsi)
+          alpha<-1-alpha
+          est.psi0<-sapply(1:ncol(Z),function(i)quantile(Z[,i],probs=1-qpsi[i],names=FALSE))
+        }
+        
+        #}
+        PSI <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
+        if(jt) Z<-apply(Z.orig,2,jitter)
+        if(nonParam){
               id<-sample(n, size=size.boot, replace=TRUE)
               o.boot<-try(suppressWarnings(seg.lm.fit(y[id], XREG[id,,drop=FALSE], Z[id,,drop=FALSE], PSI[id,,drop=FALSE],
                 w[id], offs[id], opz.boot)), silent=TRUE)
-          } else {
+        } else {
               yy<-fitted.ok+sample(residuals(o0),size=n, replace=TRUE)
               o.boot<-try(suppressWarnings(seg.lm.fit(yy, XREG, Z.orig, PSI, weights, offs, opz.boot)), silent=TRUE)
-          }
-          if(is.list(o.boot)){
+        }
+        if(is.list(o.boot)){
             all.est.psi.boot[k,]<-est.psi.boot<-o.boot$psi
-            } else {
+        } else {
             est.psi.boot<-apply(rangeZ,2,function(r)runif(1,r[1],r[2]))
-            }
-            PSI <- matrix(rep(est.psi.boot, rep(nrow(Z), length(est.psi.boot))), ncol = length(est.psi.boot))
-            opz$h<-max(opz$h*.9, .2)
-            opz$it.max<-opz$it.max+1
-            o<-try(suppressWarnings(seg.lm.fit(y, XREG, Z.orig, PSI, w, offs, opz, return.all.sol=TRUE)), silent=TRUE)
-            if(!is.list(o) && random){
+        }
+        #if(k==7) browser()
+        ### se est.psi.boot non e' cambiato (e puoi vederlo da all.est.psi.boot), allora cambialo!
+        
+        
+        PSI <- matrix(rep(est.psi.boot, rep(nrow(Z), length(est.psi.boot))), ncol = length(est.psi.boot))
+        opz$h<-max(opz$h*.9, .2)
+        opz$it.max<-opz$it.max+1
+        o<-try(suppressWarnings(seg.lm.fit(y, XREG, Z.orig, PSI, w, offs, opz, return.all.sol=TRUE)), silent=TRUE)
+        if(!is.list(o) && random){
                 est.psi0<-apply(rangeZ,2,function(r)runif(1,r[1],r[2]))
                 PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
                 o<-try(suppressWarnings(seg.lm.fit(y, XREG, Z, PSI1, w, offs, opz1)), silent=TRUE)
                 count.random<-count.random+1
-              }
-            #se il modello e' stato stimato controlla se la soluzione e' migliore..
-            if(is.list(o)){
+        }
+        #se il modello e' stato stimato controlla se la soluzione e' migliore..
+        if(is.list(o)){
               if(!"coefficients"%in%names(o$obj)) o<-extract.psi(o)
               all.est.psi[k,]<-o$psi
               all.ss[k]<-o$SumSquares.no.gap
-              if(o$SumSquares.no.gap<=ifelse(is.list(o0), o0$SumSquares.no.gap, 10^12)) o0<-o
+              if(o$SumSquares.no.gap<=ifelse(is.list(o0), o0$SumSquares.no.gap, 10^12)) {o0<-o; k.psi.change<- k}
               est.psi0<-o0$psi
               all.selected.psi[k,] <- est.psi0
               all.selected.ss[k]<-o0$SumSquares.no.gap #min(c(o$SumSquares.no.gap, o0$SumSquares.no.gap))
-              }
-
-
-            if (visualBoot) {
+        }
+            
+        if(visualBoot) {
               flush.console()
               #      spp <- if (it < 10) " " else NULL
               #      cat(paste("iter = ", spp, it,
@@ -124,21 +141,26 @@ extract.psi<-function(lista){
                         "  n.psi = ",formatC(length(unlist(est.psi0)),digits=0,format="f"), 
                         "  est.psi = ",paste(formatC(unlist(est.psi0),digits=3,format="f"), collapse="  "), #sprintf('%.2f',x)
                         sep=""), "\n")
-            }
-            
-            } #end n.boot
+        }
+        #conta i valori ss uguali.. cosi puoi fermarti prima..
+        asss<-na.omit(all.selected.ss)
+        if(length(asss)>break.boot){
+          if(all(rev(round(diff(asss),6))[1:(break.boot-1)]==0)) break
+        }
+        #id.uguali<-(round(diff(all.selected.ss[c(k-1,k-2)]),6)==0)+id.uguali      
+        } #end n.boot
 
       all.selected.psi<-rbind(est.psi00,all.selected.psi)
       all.selected.ss<-c(ss00, all.selected.ss)
 
-      SS.ok<-min(all.selected.ss)
-      id.accept<- ((abs(all.ss-SS.ok)/SS.ok )<= 0.05)
-      psi.mean<-apply(all.est.psi[id.accept,,drop=FALSE], 2, mean)
-      
-#      est.psi0<-psi.mean
-#      #devi ristimare il modello con psi.mean
-#      PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
-#      o0<-try(seg.lm.fit(y, XREG, Z, PSI1, w, offs, opz1), silent=TRUE)
+
+      # SS.ok<-min(all.selected.ss)
+      # id.accept<- ((abs(all.ss-SS.ok)/SS.ok )<= 0.05)
+      # psi.mean<-apply(all.est.psi[id.accept,,drop=FALSE], 2, mean)
+      # est.psi0<-psi.mean
+      # devi ristimare il modello con psi.mean
+      # PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
+      # o0<-try(seg.lm.fit(y, XREG, Z, PSI1, w, offs, opz1), silent=TRUE)
 
       
 
@@ -146,9 +168,10 @@ extract.psi<-function(lista){
 
       if(is.null(o0$obj)){
           PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
-          o0<-try(seg.lm.fit(y, XREG, Z, PSI1, w, offs, opz1), silent=TRUE)
+          o0<-try(suppressWarnings(seg.lm.fit(y, XREG, Z, PSI1, w, offs, opz1)), silent=TRUE)
       }
       if(!is.list(o0)) return(0)
       o0$boot.restart<-ris
+      rm(.Random.seed, envir=globalenv())
       return(o0)
       }
\ No newline at end of file
